
local myname, mm = ...
local _G = getfenv(0)

local ipairs	= _G.ipairs
local pairs		= _G.pairs
local select	= _G.select
local tinsert	= _G.tinsert
local wipe		= _G.wipe

local GetProfessionInfo	= _G.GetProfessionInfo
local GetProfessions		= _G.GetProfessions
local IsSpellKnown		= _G.IsSpellKnown
local C_MountJournal		= _G.C_MountJournal

local err = mm.err

-- Support for Legion
local GetDisplayedMountInfo, GetDisplayedMountInfoExtra, GetNumDisplayedMounts
if select(4, _G.GetBuildInfo()) >= 70000 then
	GetDisplayedMountInfo		= _G.C_MountJournal.GetDisplayedMountInfo
	GetDisplayedMountInfoExtra	= _G.C_MountJournal.GetDisplayedMountInfoExtra
	GetNumDisplayedMounts		= _G.C_MountJournal.GetNumDisplayedMounts
else
	GetDisplayedMountInfo		= _G.C_MountJournal.GetMountInfo
	GetDisplayedMountInfoExtra	= _G.C_MountJournal.GetMountInfoExtra
	GetNumDisplayedMounts		= _G.C_MountJournal.GetNumMounts
end


mm.defaults = {
	profile = {
		weight_for_nonfevarite = .05,
		weight_for_class_mounts = 1
	},
}
mm.defaultsPC = {
	weight_for_nonfevarite = .05,
}
mm.defaultsSession = {
	enabled = true,
	-- All three tables will always be in sync i.e. position ml_id.g[n], ml_name.g[n] and ml_weight[n] are for the same mount
	-- Table of mount spellID by type
	ml_id = {
		g = {}, 
		f = {}, 
		fg = {},
		ww = {}, 
		uw = {}, 
		tfg = {}, 
		tff = {}, 
		r = {},
		vashjir = {},
		ahnqiraj = {},
	},
	-- Table of mount name by type
	ml_name = {
		g = {}, 
		f = {}, 
		fg = {},
		ww = {}, 
		uw = {}, 
		tfg = {}, 
		tff = {}, 
		r = {},
		vashjir = {},
		ahnqiraj = {},
	},
	-- Table of isFavorite value by type
	ml_weight = {
		g = {}, 
		f = {}, 
		fg = {},
		ww = {}, 
		uw = {}, 
		tfg = {}, 
		tff = {}, 
		r = {},
		vashjir = {},
		ahnqiraj = {},
	},
	is_a_mount = {}
}


function mm:InitDB()
	--_G[myname.."DB"] = _G.setmetatable(_G[myname.."DB"] or {}, {__index = mm.defaults})
	--mm.sessionDB = _G[myname.."DB"]
	--_G[myname.."DB"] = {}
	-- Init AceDB 
	self.acedb = _G.LibStub("AceDB-3.0"):New(myname.."DB", self.defaults, true)
	self.db = self.acedb.profile

	--_G[myname.."DBPC"] = _G.setmetatable(_G[myname.."DBPC"] or {}, {__index = mm.defaultsPC})
	--mm.dbpc = _G[myname.."DBPC"]
	-- Init character specific saved variables
	self.dbpc = self.db.char

	-- Init sessions variables, will be discarded when the player logs out
	self.sessionDB = self.defaultsSession
end


function mm:FlushDB()
	--for i,v in pairs(mm.defaults) do if mm.sessionDB[i] == v then mm.sessionDB[i] = nil end end
	--for i,v in pairs(mm.defaultsPC) do if mm.dbpc[i] == v then mm.dbpc[i] = nil end end
end

function mm:InitProfile()
	for k,v in pairs(self.defaults.profile) do
		self.db[k] = self.db[k] or v
	end

	for k,v in pairs(self.defaultsPC) do
		self.dbpc[k] = self.dbpc[k] or v
	end

	self:RefreshMacro()
end

--[==[	Pre-defined mount lists	]==]
-- Information from http://www.warcraftmounts.com/ and http"//www.wowhead.com

local travel_with_friend = {
	[55531] = true, 	-- Mechano-Hog
	[60425] = true, 	-- Mekgineer's Chopper
	[61425] = true, 	-- Traveler's Tundra Mammoth (Alliance)
	[61447] = true, 	-- Traveler's Tundra Mammoth (Horde)
	[61465] = true, 	-- Grand Black War Mammoth (Alliance)
	[61467] = true, 	-- Grand Black War Mammoth (Horde)
	[61469] = true, 	-- Grand Ice Mammoth (Horde)
	[61470] = true, 	-- Grand Ice Mammoth (Alliance)
	[75973] = true, 	-- X-53 Touring Rocket
	[93326] = true, 	-- Sandstone Drake
	[121820] = true, 	-- Obsidian Nightwing
	[122708] = true, 	-- Grand Expedition Yak
}

local repair = {
	[61425] = true, 	-- Traveler's Tundra Mammoth (Alliance)
	[61447] = true, 	-- Traveler's Tundra Mammoth (Horde)
	[122708] = true, 	-- Grand Expedition Yak
}

-- Summonable mounts that are not listed in the mount journal
local unlisted_gound_mount_spell = {
-- spellID   isFavorite
	[23214] = true,	-- Summon Charger
	[73630] = true,	-- Summon Great Exarch's Elekk
	[69826] = true,	-- Summon Great Sunwalker Kodo
	[34767] = true,	-- Summon Thalassian Charger
	[73629] = true,	-- Summon Exarch's Elekk
	[69820] = true,	-- Summon Sunwalker Kodo
	[34769] = true,	-- Summon Thalassian Warhorse
	[13819] = true,	-- Summon Warhorse
	[23161] = true,	-- Dreadsteed
	[5784] = true,		-- Felsteed
	[48778] = true,	-- Acherus Deathcharger
}

-- Mounts that can only be used if you have a certain profession level
-- [1] == Prefession name
-- [2] == Minimum profess level needed
local profession_mounts = {
	[61309] 		= { "Tailoring", 300 }, 		-- Magnificent Flying Carpet
	[61451] 		= { "Tailoring", 300 }, 		-- Flying Carpet
	[75596]		= { "Tailoring", 300 }, 		-- Frosty Flying Carpet
	[169952] 	= { "Tailoring", 300 }, 		-- Creeping Carpet
	[171844]		= { "Leatherworking", 300 }, 	-- Dustmane Direwolf
	[44153]		= { "Engineering", 300 },		-- Flying Machine
	[44151]		= { "Engineering", 300 },		-- Turbo-Charged Flying Machine
}

-- Cache the value for GetProfessionInfo()
local profession_level = {}

-- Riding skills with
local riding_skills = {
-- spellID   min level
	[90265]		= 80,		-- Master Riding
	[34091]		= 70,		-- Artisan Riding
	[34090]		= 60,		-- Expert Riding
	[33391]		= 40,		-- Journeyman Riding
	[33388]		= 20,		-- Apprentice Riding
}

function mm:InitProfessionIndex()
	wipe(profession_level)
	for _, index in ipairs({GetProfessions()}) do
		local name, _, skillLevel = GetProfessionInfo(index)
		profession_level[name] = skillLevel
	end
end

local know_riding_skill
function mm:ExtraCheck(spellID, mountType)
	-- Some custom check for mounts that are not available to all toons but are showing as usable by the API
	local can_mount = true

	-- Profession mounts
	if profession_mounts[spellID] then
		if not profession_level[profession_mounts[spellID][1]] or
		   profession_level[profession_mounts[spellID][1]] < profession_mounts[spellID][2] then
		   can_mount = false
		end
	end

	-- Also validate that the user knows one of the riding skills
	-- 284 is the Chauffered Chopper or Mechano-Hug. It doesn't require any ridding skill.
	return can_mount and (mountType == 284 or know_riding_skill)
end

--[==[
	
	Gather the lists of mounts
	--------------------------

	g 			= Ground
	f 			= Flying
	fg			= Ground + Flying that can also be used on the ground
	ww 		= Water Walking
	uw 		= Under Water
	tff 		= Transport Friend while Flying
	tfg		= Transport Friend on the ground
	r 			= Repair
	vashjir	= Can only be used in Vashj'Ir while swimming
	ahnqiraj	= Can only be used in the Temple of Ahn'Qiraj
	
  ]==]

function mm:RefreshMountList()
	local ml_id, ml_name, ml_weight, is_a_mount = self.sessionDB.ml_id, self.sessionDB.ml_name, self.sessionDB.ml_weight, self.sessionDB.is_a_mount
	local faction_name = _G.UnitFactionGroup("player")
	local faction_id =  (faction_name == "Horde" and 0) or (faction_name == "Alliance" and 1) or 2
	local level = _G.UnitLevel("player")

	self.sessionDB.mountbytype = {}
	for k,v in pairs(ml_id) do wipe(v) end
	for k,v in pairs(ml_name) do wipe(v) end
	for k,v in pairs(ml_weight) do wipe(v) end

	-- Check if the current toon knows any ridding skills
	know_riding_skill = false
	for riding_skill in pairs(riding_skills) do
		if IsSpellKnown(riding_skill) then
			know_riding_skill = true
			break
		end
	end

	-- for i=1, GetNumDisplayedMounts() do
	for i, mountID in ipairs(C_MountJournal.GetMountIDs()) do
		local creatureName, spellID, icon, active, isUsable, sourceType, isFavorite, isFactionSpecific, faction, hideOnChar, isCollected = C_MountJournal.GetMountInfoByID(mountID)
		local creatureDisplayID, descriptionText, sourceText, isSelfMount, mountType = C_MountJournal.GetMountInfoExtraByID(mountID)

		-- local isFavorite = _G.C_MountJournal.GetIsFavorite(i)

		-- Use the spell name instead of the creature name since it sometimes differ
		if spellID then
			creatureName = _G.GetSpellInfo(spellID)
		end

		-- if creatureName == "Cindermane Charger" then
		-- 	err("creatureName = %s, spellID = %s, active = %s, isUsable = %s, sourceType = %s, isFavorite = %s, isFactionSpecific = %s, faction = %s, hideOnChar = %s, isCollected = %s, mountType = %s",creatureName, spellID, active, isUsable, sourceType, isFavorite, isFactionSpecific, faction, hideOnChar, isCollected, mountType)
		-- end

		if isCollected and not hideOnChar and (not isFactionSpecific or faction == faction_id) and self:ExtraCheck(spellID, mountType) then
			-- Remember all the valid mounts
			is_a_mount[spellID] = true

			-- The weight is 1 for favorite mount, weight_for_class_mounts for class mount and
			-- weight_for_nonfevarite for all the other mounts
			local weight = (isFavorite and 1) or 
								(unlisted_gound_mount_spell[spellID] and self.db.weight_for_class_mounts) or
								self.db.weight_for_nonfevarite

			if 		mountType == 230 then
				-- Ground mount
				tinsert(ml_id.g, spellID)
				tinsert(ml_name.g, creatureName)
				tinsert(ml_weight.g, weight)
				tinsert(ml_id.fg, spellID)
				tinsert(ml_name.fg, creatureName)
				tinsert(ml_weight.fg, weight)
			elseif 	mountType == 231 then
				-- Swim speed bonus mount
				tinsert(ml_id.uw, spellID)
				tinsert(ml_name.uw, creatureName)
				tinsert(ml_weight.uw, weight)
			elseif 	mountType == 254 then
				-- Swim speed bonus but not usable on land
				tinsert(ml_id.uw, spellID)
				tinsert(ml_name.uw, creatureName)
				tinsert(ml_weight.uw, weight)
				tinsert(ml_id.vashjir, spellID)
				tinsert(ml_name.vashjir, creatureName)
				tinsert(ml_weight.vashjir, weight)
			elseif 	mountType == 247 then
				-- Flying mount, only contains Red Flying Cloud for now, don't know why it has its own mountType
				tinsert(ml_id.f, spellID)
				tinsert(ml_name.f, creatureName)
				tinsert(ml_weight.f, weight)
				tinsert(ml_id.fg, spellID)
				tinsert(ml_name.fg, creatureName)
				tinsert(ml_weight.fg, weight)
			elseif 	mountType == 269 then
				-- Water walking mount
				tinsert(ml_id.ww, spellID)
				tinsert(ml_name.ww, creatureName)
				tinsert(ml_weight.ww, weight)
			elseif 	mountType == 248 then
				-- Flying mount that can also be used a ground mount
				tinsert(ml_id.f, spellID)
				tinsert(ml_name.f, creatureName)
				tinsert(ml_weight.f, weight)
				tinsert(ml_id.fg, spellID)
				tinsert(ml_name.fg, creatureName)
				tinsert(ml_weight.fg, weight)
			elseif 	mountType == 241 then
				-- Ground mount that can only be used in the Temple of Ahn'Qiraj
				-- (The API is lying, the red one is faster)
				tinsert(ml_id.ahnqiraj, spellID)
				tinsert(ml_name.ahnqiraj, creatureName)
				tinsert(ml_weight.ahnqiraj, weight)
			elseif 	mountType == 232 then
				-- Swiming mount that can only be used in Vashj'ir
				tinsert(ml_id.vashjir, spellID)
				tinsert(ml_name.vashjir, creatureName)
				tinsert(ml_weight.vashjir, weight)
			elseif   mountType == 284  then
				-- err("Chauffered mountType = %s, name = %s, spellID = %s, id = %s", mountType, creatureName, spellID, i)
				if level < 40 then
					-- Special mount for level 1 toon
					-- Cannot go fast so only insert it for toon level < 40
					tinsert(ml_id.g, spellID)
					tinsert(ml_name.g, creatureName)
					tinsert(ml_weight.g, weight)
					tinsert(ml_id.fg, spellID)
					tinsert(ml_name.fg, creatureName)
					tinsert(ml_weight.fg, weight)
				end
			else
				err("Unknown mountType = %s, name = %s, spellID = %s, id = %s", mountType, creatureName, spellID, i)
			end

			-- Special cases that are not specified by the API
			if travel_with_friend[spellID] then
				if mountType == 248 then
					tinsert(ml_id.tff, spellID)
					tinsert(ml_name.tff, creatureName)
					tinsert(ml_weight.tff, weight)
				elseif mountType == 230 then
					tinsert(ml_id.tfg, spellID)
					tinsert(ml_name.tfg, creatureName)
					tinsert(ml_weight.tfg, weight)
				else
					err("Unknown mountType = %s, name = %s, spellID = %s, id = %s", mountType, creatureName, spellID, i)
				end
			end

			if repair[spellID] then
				tinsert(ml_id.r, spellID)
				tinsert(ml_name.r, creatureName)
				tinsert(ml_weight.r, weight)
			end
		end

		-- For debug
		if mountType then
			self.sessionDB.mountbytype[mountType] = self.sessionDB.mountbytype[mountType] or {}
			self.sessionDB.mountbytype[mountType][("%s : %s : isSelfMount = %s : isUsable = %s : hideOnChar = %s"):format(_G.tostringall(creatureName, spellID, isSelfMount, isUsable, hideOnChar))] = true
		else
			err("mountType is nil for mount index: %s", i)
		end
	end

	-- Special case for Warlocks with the Glyph of Nightmares
	-- if select(2, _G.UnitClass("player")) == "WARLOCK" and (is_a_mount[23161] or is_a_mount[5784]) then
	-- 	for i=1,_G.GetNumGlyphSockets() do
	-- 		if select(4,_G.GetGlyphSocketInfo(i)) == 56232 then
	-- 			if is_a_mount[23161] then	
	-- 				-- Dreadsteed			
	-- 				local name =  _G.GetSpellInfo(23161)
	-- 				tinsert(ml_id.ww, 23161)
	-- 				tinsert(ml_name.ww, name)
	-- 				tinsert(ml_weight.ww, self.db.weight_for_class_mounts)
	-- 			end
	-- 			if is_a_mount[5784] then			
	-- 				-- Felsteed	
	-- 				local name =  _G.GetSpellInfo(5784)
	-- 				tinsert(ml_id.ww, 5784)
	-- 				tinsert(ml_name.ww, name)
	-- 				tinsert(ml_weight.ww, self.db.weight_for_class_mounts)
	-- 			end
	-- 		end
	-- 	end
	-- end
end
